

def percentage(part, whole):
    """
    Compute a percentage.

    :param part: The part.
    :type part: float
    :param whole: The whole.
    :type whole: float
    :return: The computed percentage.
    """
    return 100 * float(part) / float(whole)
