#!/usr/bin/env python3

"""
Package: deepnox.third

This file is a part of python-wipbox project.

(c) 2021, Deepnox SAS.
"""

__import__('pkg_resources').declare_namespace(__name__)


import aiohttp
import arrow
import numpy
import pandas
import pydantic
import requests
import shortuuid
import yaml


aiohttp = aiohttp
arrow = arrow
# jinja2 = __import__("jinja2")
numpy = numpy
pandas = pandas
pydantic = pydantic
requests = requests
shortuuid = shortuuid
yaml = yaml


